/********************************************************************************/
/*                                                                              */
/*     Betrieb des PT - First Step!                                             */
/*                                                                              */
/*     Programm:            Betrieb eines DCF77-Empfangsmoduls am PT-First Step */
/*                                                                              */
/*     Autoren:             P.G. / B.v.B.                                       */
/*                                                                              */
/*     Zielsystem:          PT - First Step, Prozessor: AT89C51CC03 (Atmel)     */
/*                                                                              */
/*     Zusatz Hardware:     DCF77 Empfangsmodul                                 */
/*                                                                              */
/*                          DCF77 Signaleingang: Portpin P3.5!                  */
/*                                                                              */
/*     IDE/C-Compiler:      uC/51 (Wickenhuser)                                */
/*                                                                              */
/*     Letzte nderung:     18.01.2010                                          */
/*                                                                              */
/*     Datei:               DCF77.c                                             */
/*                                                                              */
/********************************************************************************/

/*** Einbinden von Include-Dateien ***/
#include <stdio.h>                      // Standard Ein-/Ausgabefunktionen
#include <at89C51cc03.h>                // CC03er-Grundregister

/*** Definition der globalen Variablen  ***/
unsigned char bit DCF @ 0xb5;           // Portpin P3.5, Zeitzeichen Eingang

/********************************************************************************/
/*  Die Funktion void seri_init(void) initialisiert die ser. Schnittstelle des  */
/*  CC03ers bei einer Taktfrequenz von 11,0592MHz mit nachfolgenden Schnitt-    */
/*  stellen Parametern: 9600 Bd, 1 Stopp-Bit, keine Paritt, asynchr. Betrieb!  */
/********************************************************************************/

void seri_init(void)
  {
    SCON=0x52;
    TMOD |=0x20;
    TH1=0xfd;                      
    TR1=1;
    TI=1;
  }

/********************************************************************************/
/*  Funktion void clear_screen(void) lscht den Terminal Bildschirm!            */
/********************************************************************************/

void clear_screen(void)
  {
    printf("\x1b\x48\x1b\x4a");
  }

/********************************************************************************/
/*** Start des Hauptprogramms! **************************************************/
/********************************************************************************/

void main (void)
  {
    unsigned char MIN_ANFANG = 0;                   // Statusvariable Minutenanfang erkannt!
    unsigned int impulse = 0;                       // Zhlvariable fr Millisekundentakte!
    unsigned char i;                                // Schleifenvariable!
    unsigned char wert[59];                         // Logikzustnde der 59 Zeitzeichen
    unsigned char min, std, tag, mon, jahr, wotag;  // Zeit- und Datumsvariable
    unsigned char p1, p2, p3;                       // Hilsvariable zur Parittsprfung

    seri_init();                    // Schnittstelle initialisieren
    clear_screen();                 // Bildschirm lschen

    printf("\n\n\n   Programm zum Betrieb eines DCF77 Empfangsmoduls!");
    printf("\n  ==================================================");
    printf("\n\n\n     Achtung, messen Sie mit einem Oszilloskop den Empfang der Zeitzeichen");
    printf("\n     -> Programm funktioniert nur bei ungestoerten Empfang!!! \n\n ");

    while(!(DCF));                  // Warte, solange am Portpin 0-Signal anliegt,
                                    // damit die nchste LOW-Zeit komplett erfasst, wird!
    while(!(MIN_ANFANG))            // Warte, bis Minutenanfang erkannt wurde!
      {
        while(DCF);                 // Warte, solange am Portpin 1-Signal anliegt!
        while(!(DCF))               // Wenn 0-Signal anliegt, dann
          {                         // zhle die Millisekundentakte
            _wait_ms(1);            // solange, bis am Zeitzeicheneingang 
            impulse++;              // wieder 1-Signal anliegt!
          }    
        printf("x");                // Sekundentakte als "Lebenszeichen" ausgeben!
        if ((impulse < 700) || ((impulse > 1000) && (impulse < 1600)) || (impulse > 2000))    
          {                                                             // Prfen, ob sinnvoller 
            printf("\n\n\n   ACHTUNG, kein oder gestoerter Empfang!");  // Wert des LOW-Impulses 
            printf("\n   Programm angehalten, Neustart mit RESET!");    // gemessen wurde, wenn
            while(1);                                                   // nicht, Textausgabe 
          }                                                             // und Programm anhalten!
        if ((impulse > 1600) && (impulse < 2000))               // Prfen, ob der Wert des LOW-
          {                                                     // Impulses den Minutenanfang 
            printf("\n\n\n *** MINUTENANFANG ***\n\n\n ");      // darstellt, wenn ja dann 
            MIN_ANFANG = 1;                                     // Textausgabe und im Programm 
          }                                                     // weiter fortfahren!!
        impulse = 0;                // Variable impulse fr nchste Zhlung wieder lschen!                   
      }

    while(1)                        // Endlosschleife zum kontinuierlichen Empfang der Zeit-
      {                             // zeichen und deren Darstellung auf dem Bildschirm! 
        p1 = 0;                     // Hilfsvariablen zur Parity-Berechnung auf 0 setzen!
        p2 = 0;
        p3 = 0;
        for(i=0; i<59; i++)         // Schleife fr die 59 Informationsbits!    
          {         
            while(!(DCF));          // Warte, bis HIGH-Signal am Portpin anliegt!               
            while(DCF)              // Wenn HIGH-Signal am Portpin anliegt, 
              {                     // dann zhle die Millisekundentakte solange,
                _wait_ms(1);        // bis am Zeitzeichenengang wieder LOW-Signal anliegt!
                impulse++;         
              }
            if ((impulse > 40) && (impulse < 120)) wert[i] = 0;     // Zeitzeichen entspricht der log. 0! 
            if ((impulse > 150) && (impulse < 220)) wert[i] = 1;    // Zeitzeichen entspricht der log. 1!
            if ((impulse < 40) || ((impulse >= 120) && (impulse <= 150)) || (impulse > 220))    // Wenn 
              {                                                                 // die ermittelte HIGH-
                printf("\n\n\n   ACHTUNG, kein oder gestoerter Empfang!");      // Signalzeit kein gl- 
                printf("\n   Programm angehalten, Neustart mit RESET!");        // tiges Ergebnis dar-
                while(1);                                                       // stellt, dann Textausgabe    
              }                                                                 // und Programm anhalten!
            impulse = 0;            // Variable impulse fr nchste Zhllung wieder lschen!                              
            printf("%u", wert[i]);                                          // Das Zeitzeichen und nach jedem 
            if((i==9)||(i==19)||(i==29)||(i==39)||(i==49)) printf(" ");     // 10. Zeitzeichen ein Leerzeichen       
          }                                                                 // auf dem Terminal Bildschirm ausgeben!

        for(i=21; i<=28; i++) p1=p1+wert[i];    // Aufsummierung der Datenbits 21-28
        p1 = p1 % 2;                            // und Prfung auf gerade Paritt (p1=0)!
        for(i=29; i<=35; i++) p2=p2+wert[i];    // Aufsummierung der Datenbits 29-35
        p2 = p2 % 2;                            // und Prfung auf gerade Paritt (p2=0)!
        for(i=36; i<=58; i++) p3=p3+wert[i];    // Aufsummierung der Datenbits 36-58
        p3 = p3 % 2;                            // und Prfung auf gerade Paritt (p3=0)!
        
        if(p1+p2+p3)                            // Wenn eine Parittsprfung falsch war,
          {                                     // dann Textausgabe und Programm anhalten!       
            printf("\n\n\n   ACHTUNG, Paritaetsfehler erkannt!");       
            printf("\n   Programm angehalten, Neustart mit RESET!");            
            while(1);                                                           
          }                                                                     
                                                // Berechnung der Infos und Datenausgabe
        min=wert[21]+2*wert[22]+4*wert[23]+8*wert[24]+10*wert[25]+20*wert[26]+40*wert[27];
        std=wert[29]+2*wert[30]+4*wert[31]+8*wert[32]+10*wert[33]+20*wert[34];
        tag=wert[36]+2*wert[37]+4*wert[38]+8*wert[39]+10*wert[40]+20*wert[41];
        mon=wert[45]+2*wert[46]+4*wert[47]+8*wert[48]+10*wert[49];
        jahr=wert[50]+2*wert[51]+4*wert[52]+8*wert[53]+10*wert[54]+20*wert[55]+40*wert[56]+80*wert[57];
        wotag=wert[42]+2*wert[43]+4*wert[44];
        switch (wotag)
          {
            case 1: printf("\n\n *** Montag");
                    break;
            case 2: printf("\n\n *** Dienstag");
                    break;
            case 3: printf("\n\n *** Mittwoch");
                    break;
            case 4: printf("\n\n *** Donnerstag");
                    break;
            case 5: printf("\n\n *** Freitag");
                    break;
            case 6: printf("\n\n *** Samstag");
                    break;
            case 7: printf("\n\n *** Sonntag");
                    break;
          }
        printf(", der %02u.%02u.%02u, %02u:%02u Uhr ***\n\n\n ", tag, mon, jahr, std, min);
      }    
  }


/*******************************************************************************************/
/*** Ende des Hauptprogramms, d.h. Ende des gesamten Programms! ****************************/
/*******************************************************************************************/
