/********************************************************************************/
/*                                                                              */
/*     Betrieb des PT - First Step!                                             */
/*                                                                              */
/*     Programm:            Messung von Frequenzen mittels Timer 2!             */
/*                                                                              */
/*     Autoren:             P.G. / B.v.B.                                       */
/*                                                                              */
/*     Zielsystem:          PT - First Step, Prozessor: AT89C51CC03 (Atmel)     */
/*                                                                              */
/*     IDE/C-Compiler:      uC/51 (Wickenhuser)                                */
/*                                                                              */
/*     Letzte nderung:     13.01.2010                                          */
/*                                                                              */
/*     Datei:               FREQUENZ.c                                          */
/*                                                                              */
/********************************************************************************/

/*** Einbinden von Include-Dateien ***/
#include <stdio.h>                      // Standard Ein-/Ausgabefunktionen
#include <at89C51cc03.h>                // CC03er-Grundregister

/*** Definition der globalen Variablen  ***/
unsigned char bit IMP  @  0x90;         // Port-Pin P1.0 Frequenzeingang (T2)

/********************************************************************************/
/*  Die Funktion void seri_init(void) initialisiert die ser. Schnittstelle des  */
/*  CC03ers bei einer Taktfrequenz von 11,0592MHz mit nachfolgenden Schnitt-    */
/*  stellen Parametern: 9600 Bd, 1 Stopp-Bit, keine Paritt, asynchr. Betrieb!  */
/********************************************************************************/

void seri_init(void)
  {
    SCON=0x52;
    TMOD |=0x20;
    TH1=0xfd;                      
    TR1=1;
    TI=1;
  }

/********************************************************************************/
/*  Funktion void clear_screen(void) lscht den Terminal Bildschirm!            */
/********************************************************************************/

void clear_screen(void)
  {
    printf("\x1b\x48\x1b\x4a");
  }

/********************************************************************************/
/*  Funktion void set_cursor(unsigned char z, unsigned char s) positioniert den */
/*  Terminal Cursor auf die angegebene Position:                                */
/*  Parameter z := Zeile, Parameter s := Spalte                                 */
/********************************************************************************/

void set_cursor(unsigned char z, unsigned char s)
  {
    printf("\x1b\x59%c%c",(32+z),(32+s));
  }  

/********************************************************************************/
/*** Start des Hauptprogramms! **************************************************/
/********************************************************************************/

void main(void)
  {
    unsigned int wert;                                  // Hilfsvariable Zhlerstand der Impulsbreite
    float breite;                                       // Umgerechnete Impulsbreite
    float TMZ;                                          // Variable Wert fr den Maschinen-Zyklus
    float freq;                                         // Frequenz des Signals
    TMZ = (1.0 / 11.0592) * 12.0;                       // Maschinen-Zyklus Zeit ( = 12 Takte) in s bei Einsatz 
                                                        // eines 11.0592 MHz Quarzes. Impulse dieser Breite 
                                                        // werden vom Timer T2 gezhlt!
    seri_init();                                        // Initialisierung der ser. Schnittstelle
    clear_screen();                                     // Bildschirm lschen

    printf(" Programm Frequenzmesser! \n");              // Textmeldungen und Bildschirmmaske auf Hyperterm
    printf("==========================\n\n");
    printf("   Programm laeuft ... \n\n\n\n\n");

    printf(" Zaehlerstand T2:\n");
    printf(" Impuls-Breite:            ms\n");
    printf(" Frequenz:                 kHz");

    C_T2=0;                                             // Initialisierung der SFRs von Timer/Counter T2: 
    T2MOD=0x00;                                         // Timer Betrieb!

    while(1)                                            // Endlosschleife!      
      {
        TL2 = 0;                                        // Zhlregister lschen
        TH2 = 0;

        while(IMP);                                     // Warten auf den korrekten Impulsanfang, zuerst warten, 
        while(!(IMP));                                  // bis log. 0 - Signal am Portpin anliegt, dann auf
                                                        // Steigende Flanke warten!
        TR2 = 1;                                        // Timer 2 starten und
        while(IMP);                                     // warten, solange log. 1 - Signal am Portpin anliegt!
        TR2 = 0;                                        // Bei fallender Flanke Timer 2 wieder stoppen und
        wert = (256*TH2) + TL2;                         // Zhlerstand fr Impuslbreite tp aus den Zhlregister lesen!
        breite = (wert * TMZ) / 1000;                   // Impulsbreiten-Wert umrechnen in korrekte Impuls-Breite  
        freq = (1.0 / (breite * 2.0));                  // in ms und daraus die Frequenz in kHz berechnen!

        set_cursor(8,19);                               // Messwerte ausgeben
        printf("%7u",wert);
        set_cursor(9,19);                                
        printf("%7.3f",breite);
        set_cursor(10,19);                                
        printf("%7.3f",freq);

        _wait_ms(10);                                   // 10 ms warten, bis zur nchsten Messung!
      }
  }

/*******************************************************************************************/
/*** Ende des Hauptprogramms, d.h. Ende des gesamten Programms! ****************************/
/*******************************************************************************************/