/********************************************************************************/
/*                                                                              */
/*     Betrieb des PT - First Step!                                             */
/*                                                                              */
/*     Programm:            Programm Interruptgesteuertes erfassen von          */
/*                          5 Zaehlimpulsen am externen Countereingang T2!      */
/*                                                                              */
/*     Autoren:             P.G. / B.v.B.                                       */
/*                                                                              */
/*     Zielsystem:          PT - First Step, Prozessor: AT89C51CC03 (Atmel)     */
/*                                                                              */
/*     IDE/C-Compiler:      uC/51 (Wickenhuser)                                */
/*                                                                              */
/*     Letzte nderung:     18.01.2010                                          */
/*                                                                              */
/*     Datei:               INTER.c                                             */
/*                                                                              */
/********************************************************************************/

/*** Einbinden von Include-Dateien ***/
#include <stdio.h>                      // Standard Ein-/Ausgabefunktionen
#include <at89C51cc03.h>                // CC03er-Grundregister
#include <irq52.h>                      // Festlegung der IR-Vektoradressen



/*** Definition der globalen Variablen  ***/
unsigned char merker = 0;               // Globale Variable, merker = 1 wnn 5 Impulse gezhlt!

IRQ_VECTOR(zaehl_inter, TIMER2);        // Zuordnung der Interrupt Funktion zaehl_inter() zur
                                        // Interupt-Vektor-Adresse SERIAL!

/********************************************************************************/
/*  Die Funktion void seri_init(void) initialisiert die ser. Schnittstelle des  */
/*  CC03ers bei einer Taktfrequenz von 11,0592MHz mit nachfolgenden Schnitt-    */
/*  stellen Parametern: 9600 Bd, 1 Stopp-Bit, keine Paritt, asynchr. Betrieb!  */
/********************************************************************************/

void seri_init(void)
  {
    SCON=0x52;
    TMOD |=0x20;
    TH1=0xfd;                      
    TR1=1;
    TI=1;
  }

/********************************************************************************/
/*  Funktion void clear_screen(void) lscht den Terminal Bildschirm!            */
/********************************************************************************/

void clear_screen(void)
  {
    printf("\x1b\x48\x1b\x4a");
  }

/********************************************************************************************/
/*** Interupt Funktion: Interupt des Timer/Counter T2 des CC03ers ***************************/
/********************************************************************************************/

void zaehl_inter(void) interrupt
  {
    if(TF2)                     // Trat ein Empfangsinterrupt auf, wenn ja dann:
      {
        TF2=0;                      // TF2 softwaremig zurcksetzen,
        merker = 1;                 // Merker fr 5 Zhlimpulse empfangen setzen!
      }
  }

/********************************************************************************/
/*** Start des Hauptprogramms! **************************************************/
/********************************************************************************/

void main (void)                // Start des Hauptprogramms
  {
    seri_init();                                            // Initialisierung der ser. Schnittstelle
    clear_screen();                                         // Bildschirm lschen

    printf(" Programm Interruptgesteuertes erfassen von 5 Zaehlimpulsen!\n");
    printf("=============================================================\n\n");
    printf("   Bei jedem 5. Zaehlimpuls am externen Eingang der\n"); 
    printf("   Timer/Counter T2 Einheit wird ein Interrupt ausgeloest!\n\n\n"); 
    printf("   Programm laeuft ... \n\n\n\n\n");
                                                            // Initialisierung von Timer 2
    T2MOD=0x00;                                             // Mode-Reg. setzen
    C_T2=1;                                                 // Timer 2 als Counter

    TL2=0xfb;                                               // Zhlregister so setzen, dass nach
    TH2=0xff;                                               // 5 Zhlimpulsen ein berlauf Interrupt auftritt!

    RCAP2L=0xfb;                                            // Reloadregister so setzen, dass nach
    RCAP2H=0xff;                                            // 5 Zhlimpulsen ein berlauf Interrupt auftritt!
    
    TR2=1;                                                  // Timer/Counter 2 starten und
                                                            // Startmeldung auf Hyperterminal ausgeben!
    ET2 = 1;                                                // Interupt des Timer/Counter T2 des CC03ers freigeben!
    EA = 1;                                                 // Alle Interrupts freigeben!

    while(1)                                                // Endlosschleife!      
      {
        printf("Hallo! ");                                      // Text ausgeben! 
        if (merker == 1)                                        // Ist ein Interrupt aufgetreten?
          {                                                     // Wenn ja, dann:
            merker = 0;                                         // Merker wieder auf 0 setzen und Text ausgeben!
            printf("\n\n     *** 5 Zaehlimpulse erfasst! ***\n\n");
          }
        _wait_ms(100);                                          // 100ms bis zum nchsten Hallo warten!       
      }                              
  }

/*******************************************************************************************/
/*** Ende des Hauptprogramms, d.h. Ende des gesamten Programms! ****************************/
/*******************************************************************************************/
