/********************************************************************************/
/*                                                                              */
/*     Betrieb des PT - First Step!                                             */
/*                                                                              */
/*     Programm:            Betrieb der 4 LEDs und des Piezo-Summers!           */
/*                                                                              */
/*     Autoren:             P.G. / B.v.B.                                       */
/*                                                                              */
/*     Zielsystem:          PT - First Step, Prozessor: AT89C51CC03 (Atmel)     */
/*                                                                              */
/*     IDE/C-Compiler:      uC/51 (Wickenhuser)                                */
/*                                                                              */
/*     Letzte nderung:     12.01.2010                                          */
/*                                                                              */
/*     Datei:               LED_SUMMER.c                                        */
/*                                                                              */
/********************************************************************************/

/*** Einbinden von Include-Dateien ***/
#include <stdio.h>                      // Standard Ein-/Ausgabefunktionen
#include <at89C51cc03.h>                // CC03er-Grundregister

/*** Definition der globalen Variablen  ***/
unsigned char bit LED_1     @   0x92;   // Portpin P1.2, LED 1
unsigned char bit LED_2     @   0x93;   // Portpin P1.3, LED 2
unsigned char bit LED_3     @   0x94;   // Portpin P1.4, LED 3  
unsigned char bit LED_4     @   0x95;   // Portpin P1.5, LED 4
unsigned char bit SUMMER    @   0x91;   // Portpin P1.1, SUMMER

/********************************************************************************/
/*  Die Funktion void seri_init(void) initialisiert die ser. Schnittstelle des  */
/*  CC03ers bei einer Taktfrequenz von 11,0592MHz mit nachfolgenden Schnitt-    */
/*  stellen Parametern: 9600 Bd, 1 Stopp-Bit, keine Paritt, asynchr. Betrieb!  */
/********************************************************************************/

void seri_init(void)
  {
    SCON=0x52;
    TMOD |=0x20;
    TH1=0xfd;                      
    TR1=1;
    TI=1;
  }

/********************************************************************************/
/*  Funktion void clear_screen(void) lscht den Terminal Bildschirm!            */
/********************************************************************************/

void clear_screen(void)
  {
    printf("\x1b\x48\x1b\x4a");
  }

/********************************************************************************/
/*** Start des Hauptprogramms! **************************************************/
/********************************************************************************/

void main(void)
  {
    seri_init();                                        // Initialisierung der ser. Schnittstelle
    clear_screen();                                     // Bildschirm lschen

    printf(" Betrieb der LEDs und des Summers! \n");    // Textmeldungen auf Hyperterm
    printf("===================================\n\n");
    printf("   Programm laeuft ... ");

    while(1)                                            // Endlosschleife                                            
      {
        LED_1 = 0;                                          // LED 1 einschalten
        _wait_ms(250);                                      // 0,25 Sekunden warten
        LED_2 = 0;                                          // LED 2 einschalten
        _wait_ms(250);                                      // 0,25 Sekunden warten                                     
         LED_3 = 0;                                         // LED 3 einschalten
        _wait_ms(250);                                      // 0,25 Sekunden warten                                      
         LED_4 = 0;                                         // LED 4 einschalten
        _wait_ms(250);                                      // 0,25 Sekunden warten                                      
         LED_1 = 1;                                         // LED 1 ausschalten
        _wait_ms(250);                                      // 0,25 Sekunden warten
         LED_2 = 1;                                         // LED 2 ausschalten
        _wait_ms(250);                                      // 0,25 Sekunden warten
         LED_3 = 1;                                         // LED 3 ausschalten
        _wait_ms(250);                                      // 0,25 Sekunden warten
         LED_4 = 1;                                         // LED 4 ausschalten
        _wait_ms(250);                                      // 0,25 Sekunden warten
        SUMMER = 0;                                         // SUMMER einschalten
        _wait_ms(50);                                       // 0,05 Sekunden warten
        SUMMER = 1;                                         // SUMMER ausschalten
      }
  }

/*******************************************************************************************/
/*** Ende des Hauptprogramms, d.h. Ende des gesamten Programms! ****************************/
/*******************************************************************************************/
