/********************************************************************************/
/*                                                                              */
/*     Betrieb des PT - First Step!                                             */
/*                                                                              */
/*     Programm:            Ausgabe von Rechteckschwingungen mit variabler      */
/*                          Frequenz mittels Timer 2!                           */
/*                                                                              */
/*     Autoren:             P.G. / B.v.B.                                       */
/*                                                                              */
/*     Zielsystem:          PT - First Step, Prozessor: AT89C51CC03 (Atmel)     */
/*                                                                              */
/*     IDE/C-Compiler:      uC/51 (Wickenhuser)                                */
/*                                                                              */
/*     Letzte nderung:     13.01.2010                                          */
/*                                                                              */
/*     Datei:               RECHTECK.c                                          */
/*                                                                              */
/********************************************************************************/

/*** Einbinden von Include-Dateien ***/
#include <stdio.h>                      // Standard Ein-/Ausgabefunktionen
#include <at89C51cc03.h>                // CC03er-Grundregister

/********************************************************************************/
/*  Die Funktion void seri_init(void) initialisiert die ser. Schnittstelle des  */
/*  CC03ers bei einer Taktfrequenz von 11,0592MHz mit nachfolgenden Schnitt-    */
/*  stellen Parametern: 9600 Bd, 1 Stopp-Bit, keine Paritt, asynchr. Betrieb!  */
/********************************************************************************/

void seri_init(void)
  {
    SCON=0x52;
    TMOD |=0x20;
    TH1=0xfd;                      
    TR1=1;
    TI=1;
  }

/********************************************************************************/
/*  Funktion void clear_screen(void) lscht den Terminal Bildschirm!            */
/********************************************************************************/

void clear_screen(void)
  {
    printf("\x1b\x48\x1b\x4a");
  }

/********************************************************************************/
/*** Start des Hauptprogramms! **************************************************/
/********************************************************************************/

void main(void)
  {
    unsigned char c1;                                   // Hilfsvariable fr Tastendruck
    unsigned char merker;                               // Hilfsvariable fr Richtig/Falsch-Merker
    seri_init();                                        // Initialisierung der ser. Schnittstelle
                                                        // Initialisierung der SFRs von Timer/Counter T2 
    T2MOD = 0x02;                                       // fr den Clock Output Mode: T2 als Taktausgang
    C_T2=0;                                             // und Timer Betrieb!

    while(1)                                            // Endlosschleife!      
      {
        merker = 0;                                                 // Merker auf (Startwert) falsch setzen!
        clear_screen();                                             // Bildschirm lschen
        printf(" Programm Rechteckschwingung! \n");                 // Bildschirmmaske auf Hyperterm!
        printf("==============================\n\n");
        printf("   Programm laeuft ... \n\n\n\n\n");
        printf(" Bitte die Ausgangsfrequenz waehlen:\n\n");
        printf("   1)  100  Hz\n");
        printf("   2)    1 kHz\n");
        printf("   3)   10 kHz\n");
        printf("   4)  500 kHz\n\n\n");
        printf(" Ihre Wahl:  ");

        c1 = getchar();                                             // Taste einlesen und
        printf("%c",c1);                                            // Echo zurckgeben!
        TR2 = 0;                                                    // Timer/Counter T2 anhalten!
        
        switch(c1)                                                  // Auswertung des Tastendrucks 
          {                                                         // und Neuladen der Register:
            case '1' :  RCAP2H = 148;   RCAP2L = 0;                 // Nachlade-Werte fr 100 Hz
                        TH2 = 148;      TL2 = 0;
                        merker = 1;                                 // Zulssige Eingabe
                        break;

            case '2' :  RCAP2H = 245;   RCAP2L = 51;                // Nachlade-Werte fr 1 kHz
                        TH2 = 245;      TL2 = 51;
                        merker = 1;                                 // Zulssige Eingabe
                        break;

            case '3' :  RCAP2H = 254;   RCAP2L = 236;               // Nachlade-Werte fr 10 kHz
                        TH2 = 254;      TL2 = 236;
                        merker = 1;                                 // Zulssige Eingabe
                        break;

            case '4' :  RCAP2H = 255;   RCAP2L = 250;               // Nachlade-Werte fr 500 kHz
                        TH2 = 255;      TL2 = 250;
                        merker = 1;                                 // Zulssige Eingabe
                        break;

            default :   printf("   Leider falsche Eingabe !");      
                        printf("\n                Bitte eine Taste druecken ... ");
                        c1 = getchar();                             // Auf Tastendruck warten
                        break;
          }

        if (merker==1)                                              // Wenn Auswahl korrekt war, dann:
          {
            TR2 = 1;                                                // Timer/Counter T2 starten und 
            printf("\n\n Signal wird ausgegeben ...");              // Textmeldung auf HyperTerm!
            printf("\n\n Zur Neuauswahl bitte eine Taste druecken ... ");
            c1 = getchar();                                         // Auf Tastendruck (neue Frequenz) warten!
          }
      }
  }

/*******************************************************************************************/
/*** Ende des Hauptprogramms, d.h. Ende des gesamten Programms! ****************************/
/*******************************************************************************************/
