/********************************************************************************/
/*                                                                              */
/*     Betrieb des PT - First Step!                                             */
/*                                                                              */
/*     Programm:            Betrieb der 4-fach Schaltrelais-Karte PT-Relais!    */
/*                          Anzeige der aktivierte Relais auf HyperTerm!        */
/*                                                                              */
/*     Autoren:             P.G. / B.v.B.                                       */
/*                                                                              */
/*     Zielsystem:          PT - First Step, Prozessor: AT89C51CC03 (Atmel)     */
/*                                                                              */
/*     Zusatz Hardware:     PT - Relais                                         */
/*                                                                              */
/*                          Verbindung per 6-poliger MiniDIN-Leitung            */
/*                              Anschluss: X7 PT-FS  <->  X1 PT - Relais        */
/*                                                                              */
/*     IDE/C-Compiler:      uC/51 (Wickenhuser)                                */
/*                                                                              */
/*     Letzte nderung:     14.01.2010                                          */
/*                                                                              */
/*     Datei:               RELAIS.c                                            */
/*                                                                              */
/********************************************************************************/

/*** Einbinden von Include-Dateien ***/
#include <stdio.h>                      // Standard Ein-/Ausgabefunktionen
#include <at89C51cc03.h>                // CC03er-Grundregister

/*** Definition der globalen Variablen  ***/
unsigned char bit RELAIS_0 @ 0xb5;      // Portpin P3.5, Relais 0
unsigned char bit RELAIS_1 @ 0xb3;      // Portpin P3.3, Relais 1
unsigned char bit RELAIS_2 @ 0xb4;      // Portpin P3.4, Relais 2
unsigned char bit RELAIS_3 @ 0xb2;      // Portpin P3.2, Relais 3

/********************************************************************************/
/*  Die Funktion void seri_init(void) initialisiert die ser. Schnittstelle des  */
/*  CC03ers bei einer Taktfrequenz von 11,0592MHz mit nachfolgenden Schnitt-    */
/*  stellen Parametern: 9600 Bd, 1 Stopp-Bit, keine Paritt, asynchr. Betrieb!  */
/********************************************************************************/

void seri_init(void)
  {
    SCON=0x52;
    TMOD |=0x20;
    TH1=0xfd;                      
    TR1=1;
    TI=1;
  }

/********************************************************************************/
/*  Funktion void clear_screen(void) lscht den Terminal Bildschirm!            */
/********************************************************************************/

void clear_screen(void)
  {
    printf("\x1b\x48\x1b\x4a");
  }

/********************************************************************************/
/*  Funktion void set_cursor(unsigned char z, unsigned char s) positioniert den */
/*  Terminal Cursor auf die angegebene Position:                                */
/*  Parameter z := Zeile, Parameter s := Spalte                                 */
/********************************************************************************/

void set_cursor(unsigned char z, unsigned char s)
  {
    printf("\x1b\x59%c%c",(32+z),(32+s));
  }  

/********************************************************************************/
/*** Start des Hauptprogramms! **************************************************/
/********************************************************************************/

void main(void)
  {
    seri_init();                                        // Initialisierung der ser. Schnittstelle
    clear_screen();                                     // Bildschirm lschen

    printf(" Betrieb des Relais-Moduls!\n");            // Textmeldungen und Bildschirmmaske auf Hyperterm
    printf("============================\n\n");
    printf("   Die Relais werden nacheinander, jeweils\n");  
    printf("   auf Tastendruck, ein- bzw ausgeschaltet!\n\n");  
    printf("   Programm laeuft ... \n\n\n\n\n");
    printf("RELAIS K");

    while(1)                                            // Endlosschleife!      
      {
        set_cursor(11,8);                                   // Cursor positionieren, Relais
        RELAIS_0 = 0;                                       // einschalten und entsprechende
        printf("0 EIN");                                    // Textmeldung auf HyperTerm!
        getchar();                                          // Auf Tastendruck warten, dann 
        set_cursor(11,8);                                   // Cursor positionieren, Relais
        RELAIS_0 = 1;                                       // ausschalten, entsprechende
        printf("0 AUS");                                    // Textmeldung auf HyperTerm und
        getchar();                                          // wieder auf Tastendruck warten!

        set_cursor(11,8);                                   // Wie oben!
        RELAIS_1 = 0;
        printf("1 EIN");
        getchar();
        set_cursor(11,8);
        RELAIS_1 = 1;
        printf("1 AUS");
        getchar();

        set_cursor(11,8);
        RELAIS_2 = 0;
        printf("2 EIN");
        getchar();
        set_cursor(11,8);
        RELAIS_2 = 1;
        printf("2 AUS");
        getchar();

        set_cursor(11,8);
        RELAIS_3 = 0;
        printf("3 EIN");
        getchar();
        set_cursor(11,8);
        RELAIS_3 = 1;
        printf("3 AUS");
        getchar();
      }
  }

/*******************************************************************************************/
/*** Ende des Hauptprogramms, d.h. Ende des gesamten Programms! ****************************/
/*******************************************************************************************/
